package net.mcreator.rsh.network;

import net.neoforged.neoforge.registries.NeoForgeRegistries;
import net.neoforged.neoforge.registries.DeferredRegister;
import net.neoforged.neoforge.network.handling.PlayPayloadContext;
import net.neoforged.neoforge.network.PacketDistributor;
import net.neoforged.neoforge.event.entity.player.PlayerEvent;
import net.neoforged.neoforge.common.util.INBTSerializable;
import net.neoforged.neoforge.attachment.AttachmentType;
import net.neoforged.fml.event.lifecycle.FMLCommonSetupEvent;
import net.neoforged.fml.common.Mod;
import net.neoforged.bus.api.SubscribeEvent;

import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.Entity;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.network.protocol.common.custom.CustomPacketPayload;
import net.minecraft.network.protocol.PacketFlow;
import net.minecraft.network.chat.Component;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.client.Minecraft;

import net.mcreator.rsh.RshMod;

import java.util.function.Supplier;

@Mod.EventBusSubscriber(bus = Mod.EventBusSubscriber.Bus.MOD)
public class RshModVariables {
	public static final DeferredRegister<AttachmentType<?>> ATTACHMENT_TYPES = DeferredRegister.create(NeoForgeRegistries.Keys.ATTACHMENT_TYPES, RshMod.MODID);
	public static final Supplier<AttachmentType<PlayerVariables>> PLAYER_VARIABLES = ATTACHMENT_TYPES.register("player_variables", () -> AttachmentType.serializable(() -> new PlayerVariables()).build());

	@SubscribeEvent
	public static void init(FMLCommonSetupEvent event) {
		RshMod.addNetworkMessage(PlayerVariablesSyncMessage.ID, PlayerVariablesSyncMessage::new, PlayerVariablesSyncMessage::handleData);
	}

	@Mod.EventBusSubscriber
	public static class EventBusVariableHandlers {
		@SubscribeEvent
		public static void onPlayerLoggedInSyncPlayerVariables(PlayerEvent.PlayerLoggedInEvent event) {
			if (event.getEntity() instanceof ServerPlayer player)
				player.getData(PLAYER_VARIABLES).syncPlayerVariables(event.getEntity());
		}

		@SubscribeEvent
		public static void onPlayerRespawnedSyncPlayerVariables(PlayerEvent.PlayerRespawnEvent event) {
			if (event.getEntity() instanceof ServerPlayer player)
				player.getData(PLAYER_VARIABLES).syncPlayerVariables(event.getEntity());
		}

		@SubscribeEvent
		public static void onPlayerChangedDimensionSyncPlayerVariables(PlayerEvent.PlayerChangedDimensionEvent event) {
			if (event.getEntity() instanceof ServerPlayer player)
				player.getData(PLAYER_VARIABLES).syncPlayerVariables(event.getEntity());
		}

		@SubscribeEvent
		public static void clonePlayer(PlayerEvent.Clone event) {
			PlayerVariables original = event.getOriginal().getData(PLAYER_VARIABLES);
			PlayerVariables clone = new PlayerVariables();
			clone.allowed = original.allowed;
			clone.nightVision = original.nightVision;
			clone.s0 = original.s0;
			clone.s1 = original.s1;
			clone.s2 = original.s2;
			clone.s3 = original.s3;
			clone.s4 = original.s4;
			clone.s5 = original.s5;
			clone.s6 = original.s6;
			clone.s7 = original.s7;
			clone.s8 = original.s8;
			if (!event.isWasDeath()) {
			}
			event.getEntity().setData(PLAYER_VARIABLES, clone);
		}
	}

	public static class PlayerVariables implements INBTSerializable<CompoundTag> {
		public boolean allowed = false;
		public boolean nightVision = false;
		public ItemStack s0 = ItemStack.EMPTY;
		public ItemStack s1 = ItemStack.EMPTY;
		public ItemStack s2 = ItemStack.EMPTY;
		public ItemStack s3 = ItemStack.EMPTY;
		public ItemStack s4 = ItemStack.EMPTY;
		public ItemStack s5 = ItemStack.EMPTY;
		public ItemStack s6 = ItemStack.EMPTY;
		public ItemStack s7 = ItemStack.EMPTY;
		public ItemStack s8 = ItemStack.EMPTY;

		@Override
		public CompoundTag serializeNBT() {
			CompoundTag nbt = new CompoundTag();
			nbt.putBoolean("allowed", allowed);
			nbt.putBoolean("nightVision", nightVision);
			nbt.put("s0", s0.save(new CompoundTag()));
			nbt.put("s1", s1.save(new CompoundTag()));
			nbt.put("s2", s2.save(new CompoundTag()));
			nbt.put("s3", s3.save(new CompoundTag()));
			nbt.put("s4", s4.save(new CompoundTag()));
			nbt.put("s5", s5.save(new CompoundTag()));
			nbt.put("s6", s6.save(new CompoundTag()));
			nbt.put("s7", s7.save(new CompoundTag()));
			nbt.put("s8", s8.save(new CompoundTag()));
			return nbt;
		}

		@Override
		public void deserializeNBT(CompoundTag nbt) {
			allowed = nbt.getBoolean("allowed");
			nightVision = nbt.getBoolean("nightVision");
			s0 = ItemStack.of(nbt.getCompound("s0"));
			s1 = ItemStack.of(nbt.getCompound("s1"));
			s2 = ItemStack.of(nbt.getCompound("s2"));
			s3 = ItemStack.of(nbt.getCompound("s3"));
			s4 = ItemStack.of(nbt.getCompound("s4"));
			s5 = ItemStack.of(nbt.getCompound("s5"));
			s6 = ItemStack.of(nbt.getCompound("s6"));
			s7 = ItemStack.of(nbt.getCompound("s7"));
			s8 = ItemStack.of(nbt.getCompound("s8"));
		}

		public void syncPlayerVariables(Entity entity) {
			if (entity instanceof ServerPlayer serverPlayer)
				PacketDistributor.PLAYER.with(serverPlayer).send(new PlayerVariablesSyncMessage(this));
		}
	}

	public record PlayerVariablesSyncMessage(PlayerVariables data) implements CustomPacketPayload {
		public static final ResourceLocation ID = new ResourceLocation(RshMod.MODID, "player_variables_sync");

		public PlayerVariablesSyncMessage(FriendlyByteBuf buffer) {
			this(new PlayerVariables());
			this.data.deserializeNBT(buffer.readNbt());
		}

		@Override
		public void write(final FriendlyByteBuf buffer) {
			buffer.writeNbt(data.serializeNBT());
		}

		@Override
		public ResourceLocation id() {
			return ID;
		}

		public static void handleData(final PlayerVariablesSyncMessage message, final PlayPayloadContext context) {
			if (context.flow() == PacketFlow.CLIENTBOUND && message.data != null) {
				context.workHandler().submitAsync(() -> Minecraft.getInstance().player.getData(PLAYER_VARIABLES).deserializeNBT(message.data.serializeNBT())).exceptionally(e -> {
					context.packetHandler().disconnect(Component.literal(e.getMessage()));
					return null;
				});
			}
		}
	}
}
